<?php
/*
*	The TS3MusicBot.net hoster API can be used for automatically license management.
*	You need a valid hoster key (secret key) to interact with the hoster API.
*	
*	Die TS3MusicBot.net Hoster API kann fr die automatische Lizenzverwaltung verwendet werden.
*	Sie bentigen einen gltigen Hoster-Schlssel (geheimer Schlssel), um mit der Hoster API zu interagieren.
*
*	TS3MusicBot.php version 0.0.2
*	Project site: http://TS3MusicBot.net
*/
class TS3MusicBotAPI {

    const HOSTER_KEY = 'xxx'; //Set your hoster key (secret key) here.
	
	
    const API_URL = 'http://ts3musicbot.net/hoster-api.php';
    private $request = array();
    private $lastResult;

	/*
	*	Create a new license.
	*	@returns a license string or false
	*/
    public function createLicense($tsHost, $tsPort) {
        $this->request = array(
            'mode' => 'createlicense',
            'teamspeakip' => $tsHost .':' . $tsPort,
        );
        $res = $this->sendRequest();
        return isset($res->response->account) ? $res->response->account : false;
    }

	/*
	*	Change the teamspeak HOST:PORT of a license.
	*	@returns true or false
	*/
    public function changeServer($license, $tsHost, $tsPort) {
        $this->request = array(
            'mode' => 'changeip',
            'ts3_musicbot_account' => $license,
            'teamspeakip' => $tsHost .':' . $tsPort,
        );
        $res = $this->sendRequest();
        return isset($res->response->status) ? $res->response->status == 'success' : false;
    }

	/*
	*	Delete a license (deletes the licensed time too!).
	*	@returns true or false
	*/
    public function deleteLicense($license) {
        $this->request = array(
            'mode' => 'deletelicense',
            'ts3_musicbot_account' => $license
        );
        $res = $this->sendRequest();
        return isset($res->response->status) ? $res->response->status == 'success' : false;
    }

	/*
	*	Save settings. $settings can be an empty array or an array with one or more of the following entries:
	*	$settings = array(
	*		'ts3_server_query_password' => 'querypassword',
	*		'ts3_server_query_login' => 'serveradmin',
	*		'ts3_server_password' => 'ts3password',
	*		'ts3_server_query_port' => '10011',
	*		'ts3_virtualserver_id' => '-1', //TS3MusicBot will connect with the port instead if no virtualserver id is set.
	*		'ts3_musicbot_name' => 'MusicBot', //Client name visible on the ts3 server.
	*		'bot_admin_list' => '',
	*		'radio_admin_list' => '',
	*		'radio_admin_server_group_id' => '',
	*		'welcomesound_enable' => '0',
	*		'welcome_sound_music_file' => '',
	*		'welcome_sound_say_message' => '',
	*		'bot_channel_id' => '-1',
	*		'bot_channel_password' => '',
	*		'youtube_proxy' => '',
	*		'autostop_channel_empty' => '0',
	*		'welcome_sound_say_language' => 'en',
	*		'bot_server_query_name' => 'TS3MusicBot',
	*		'channel_notifications_enable' => '1',
	*		'send_radio_infos' => '1',
	*		'autostart' => 'music/xyz.mp3', //file, folder or youtube link/playlist
	*		'repeat' => '0', //repeat function for autostart
	*		'initial_volume' => '30'
	*	);
	*
	*	@returns true or false
	*/
    public function saveSettings($license, $settings) {
        $this->request = array_merge(array(
            'mode' => 'savesettings',
            'ts3_musicbot_account' => $license,
        ), $settings);
        $res = $this->sendRequest();
        return isset($res->response->status) ? $res->response->status == 'success' : false;
    }
	
	/*
	*	Fetch all settings for a license
	*	@returns json_array or false
	*/
	public function getSettings($license) {
        $this->request = array(
            'mode' => 'getsettings',
            'ts3_musicbot_account' => $license,
        );
        $res = $this->sendRequest();
        return isset($res->response->status) ? false : $res;
    }
	
	/*
	*	Reset settings to default for license
	*	@returns true or false
	*/
	public function resetSettings($license) {
        $this->request = array(
            'mode' => 'resetsettings',
            'ts3_musicbot_account' => $license
        );
        $res = $this->sendRequest();
        return isset($res->response->status) ? $res->response->status == 'success' : false;
    }

	/*
	*	Create and send the complete request.
	*/
    private function sendRequest() {
        $request = array_merge(array('hosterkey' => self::HOSTER_KEY), $this->request);
        //if ($request['mode'] == 'savesettings') print_r($request);
        $options = array(
            'http' => array(
                'header' => "Content-type: application/x-www-form-urlencoded\r\n",
                'method' => 'POST',
                'content' => http_build_query($request),
            )
        );
        $context  = stream_context_create($options);
        $this->lastResult = file_get_contents(self::API_URL, false, $context);
        if (empty($this->lastResult)) {
            throw new TS3MusicBotAPIException('API call failed. Empty result.');
        }
        $data = json_decode($this->lastResult);
        if (json_last_error() !== JSON_ERROR_NONE) {
            throw new TS3MusicBotAPIException('API json_decode error: "' . $this->getJsonError() . '" RES: "'.$this->lastResult.'"');
        }
        return $data;
    }
    
	/*
	*	If a json request failed get the error
	*	@returns error message
	*/
    private function getJsonError() {
        $json_errors = array(
            JSON_ERROR_NONE => 'No error has occurred',
            JSON_ERROR_DEPTH => 'The maximum stack depth has been reached',
            JSON_ERROR_CTRL_CHAR => 'Control character error, possibly coded incorrectly',
            JSON_ERROR_SYNTAX => 'Syntax error'
        );
        return $json_errors[json_last_error()];
    }
    
    public function getLastResult() {
        return $this->lastResult;
    }
}
